@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2022 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2001-2025 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -42,7 +42,7 @@
 void
 ROJTRFrame::fillOptions() {
     OptionsCont& oc = OptionsCont::getOptions();
-    oc.addCallExample("-c <CONFIGURATION>", "run with routing options defined in file");
+    oc.addCallExample("-c <CONFIGURATION>", TL("run with routing options defined in file"));
 
     // insert options sub-topics
     SystemFrame::addConfigurationOptions(oc);
@@ -56,39 +56,39 @@ ROJTRFrame::fillOptions() {
 
     oc.doRegister("turn-ratio-files", 't', new Option_FileName());
     oc.addSynonyme("turn-ratio-files", "turns");
-    oc.addDescription("turn-ratio-files", "Input", "Read turning ratios from FILE(s)");
+    oc.addDescription("turn-ratio-files", "Input", TL("Read turning ratios from FILE(s)"));
 
     oc.doRegister("exit-times", new Option_Bool(false));
-    oc.addDescription("exit-times", "Output", "Write exit times (weights) for each edge");
+    oc.addDescription("exit-times", "Output", TL("Write exit times (weights) for each edge"));
 
     oc.doRegister("max-edges-factor", new Option_Float(2.0));
     oc.addDescription("max-edges-factor", "Processing",
                       "Routes are cut off when the route edges to net edges ratio is larger than FLOAT");
 
     oc.doRegister("turn-defaults", 'T', new Option_StringVector(StringVector({ "30", "50", "20" })));
-    oc.addDescription("turn-defaults", "Processing", "Use STR[] as default turn definition");
+    oc.addDescription("turn-defaults", "Processing", TL("Use STR[] as default turn definition"));
 
     oc.doRegister("sink-edges", new Option_StringVector());
     oc.addSynonyme("sink-edges", "sinks");
-    oc.addDescription("sink-edges", "Processing", "Use STR[] as list of sink edges");
+    oc.addDescription("sink-edges", "Processing", TL("Use STR[] as list of sink edges"));
 
     oc.doRegister("accept-all-destinations", 'A', new Option_Bool(false));
-    oc.addDescription("accept-all-destinations", "Processing", "Whether all edges are allowed as sink edges");
+    oc.addDescription("accept-all-destinations", "Processing", TL("Whether all edges are allowed as sink edges"));
 
     oc.doRegister("ignore-vclasses", 'i', new Option_Bool(false));
-    oc.addDescription("ignore-vclasses", "Processing", "Ignore road restrictions based on vehicle class");
+    oc.addDescription("ignore-vclasses", "Processing", TL("Ignore road restrictions based on vehicle class"));
 
     oc.doRegister("allow-loops", new Option_Bool(false));
-    oc.addDescription("allow-loops", "Processing", "Allow to re-use a road");
+    oc.addDescription("allow-loops", "Processing", TL("Allow to re-use a road"));
 
     oc.doRegister("sources-are-sinks", 'S', new Option_Bool(false));
-    oc.addDescription("sources-are-sinks", "Processing", "Use all source edges as sink edges.");
+    oc.addDescription("sources-are-sinks", "Processing", TL("Use all source edges as sink edges."));
 
     oc.doRegister("discount-sources", 'D',  new Option_Bool(false));
-    oc.addDescription("discount-sources", "Processing", "Subtract upstream flow when inserting a new flow. When option --sources-are-sinks is set, the upstream flow is limited to the value of the source flow and the remaining part terminates.");
+    oc.addDescription("discount-sources", "Processing", TL("Subtract upstream flow when inserting a new flow. When option --sources-are-sinks is set, the upstream flow is limited to the value of the source flow and the remaining part terminates."));
 
     // add rand options
-    RandHelper::insertRandOptions();
+    RandHelper::insertRandOptions(oc);
 }
 
 
@@ -97,7 +97,7 @@ ROJTRFrame::checkOptions() {
     OptionsCont& oc = OptionsCont::getOptions();
     bool ok = ROFrame::checkOptions(oc);
     if (oc.isDefault("no-internal-links")) {
-        oc.set("no-internal-links", "true");
+        oc.setDefault("no-internal-links", "true");
     }
 
     SUMOVehicleParameter p;
