@@ -1,26 +1,24 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2018 German Aerospace Center (DLR) and others.
-// This program and the accompanying materials
-// are made available under the terms of the Eclipse Public License v2.0
-// which accompanies this distribution, and is available at
-// http://www.eclipse.org/legal/epl-v20.html
-// SPDX-License-Identifier: EPL-2.0
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2001-2025 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 /****************************************************************************/
 /// @file    MSCFModel_Wiedemann.h
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    June 2011
-/// @version $Id$
 ///
 // The psycho-physical model of Wiedemann
 /****************************************************************************/
-#ifndef MSCFModel_Wiedemann_H
-#define MSCFModel_Wiedemann_H
-
-// ===========================================================================
-// included modules
-// ===========================================================================
+#pragma once
 #include <config.h>
 
 #include "MSCFModel.h"
@@ -65,22 +63,23 @@ public:
     /** @brief Computes the vehicle's safe speed (no dawdling)
      * @param[in] veh The vehicle (EGO)
      * @param[in] speed The vehicle's speed
-     * @param[in] gap2pred The (netto) distance to the LEADER
+     * @param[in] gap2pred The (net) distance to the LEADER
      * @param[in] predSpeed The speed of LEADER
      * @return EGO's safe speed
      * @see MSCFModel::ffeV
      */
-    double followSpeed(const MSVehicle* const veh, double speed, double gap2pred, double predSpeed, double predMaxDecel, const MSVehicle* const pred = 0) const;
+    double followSpeed(const MSVehicle* const veh, double speed, double gap2pred,
+                       double predSpeed, double predMaxDecel, const MSVehicle* const pred = 0, const CalcReason usage = CalcReason::CURRENT) const;
 
 
     /** @brief Computes the vehicle's safe speed for approaching a non-moving obstacle (no dawdling)
      * @param[in] veh The vehicle (EGO)
-     * @param[in] gap The (netto) distance to the the obstacle
+     * @param[in] gap The (net) distance to the obstacle
      * @return EGO's safe speed for approaching a non-moving obstacle
      * @see MSCFModel::ffeS
      * @todo generic Interface, models can call for the values they need
      */
-    double stopSpeed(const MSVehicle* const veh, const double speed, double gap) const;
+    double stopSpeed(const MSVehicle* const veh, const double speed, double gap, double decel, const CalcReason usage = CalcReason::CURRENT) const;
 
 
     /** @brief Returns the maximum gap at which an interaction between both vehicles occurs
@@ -92,8 +91,16 @@ public:
      * @todo evaluate signature
      * @see MSCFModel::interactionGap
      */
-    double interactionGap(const MSVehicle* const , double vL) const;
+    double interactionGap(const MSVehicle* const, double vL) const;
 
+    /** @brief Returns the minimum gap to reserve if the leader is braking at maximum (>=0)
+     * @param[in] veh The vehicle itself, for obtaining other values
+     * @param[in] pred The leader vehicle, for obtaining other values
+     * @param[in] speed EGO's speed
+     * @param[in] leaderSpeed LEADER's speed
+     * @param[in] leaderMaxDecel LEADER's max. deceleration rate
+     */
+    double getSecureGap(const MSVehicle* const veh, const MSVehicle* const pred, const double speed, const double leaderSpeed, const double leaderMaxDecel) const;
 
     /** @brief Returns the model's name
      * @return The model's name
@@ -130,14 +137,14 @@ private:
     /* @brief the main enty point for the speed computation
      * @param[in] gap The netto gap (front bumper of ego to back bumper of leader)
      */
-    double _v(const MSVehicle* veh, double predSpeed, double gap) const;
+    double _v(const MSVehicle* veh, double predSpeed, double gap, double predAccel) const;
 
     /// @name acceleration based on the 'driving regime'
     /// @{
     double fullspeed(double v, double vpref, double dx, double bx) const; // also 'WUNSCH'
     double following(double sign) const; // also 'FOLGEN'
-    double approaching(double dv, double dx, double abx) const;  // also 'BREMSBX'
-    double emergency(double dv, double dx) const; // also 'BREMSAX'
+    double approaching(double dv, double dx, double abx, double predAccel) const;  // also 'BREMSBX'
+    double emergency(double dv, double dx, double predAccel, double v, double gap, double abx, double bx) const; // also 'BREMSAX'
     /// @}
 
 private:
@@ -159,17 +166,14 @@ private:
     /// @brief The vehicle's minimum acceleration [m/s^2] // also b_null
     const double myMinAccel;
 
+    /// @brief The maximum deceleration when approaching
+    const double myMaxApproachingDecel;
+
     /// @brief free-flow distance in m
     static const double D_MAX;
     /// @}
 
-    /// @brief vsafe from krauss since Wiedemann is deficient at approaching
-    // standing obstacles (see MSCFModel_Krauss::_vsafe)
-    double krauss_vsafe(double gap, double predSpeed) const;
-
 private:
     /// @brief Invalidated assignment operator
     MSCFModel_Wiedemann& operator=(const MSCFModel_Wiedemann& s);
 };
-
-#endif /* MSCFModel_Wiedemann_H */
