@@ -1,6 +1,6 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2007-2022 German Aerospace Center (DLR) and others.
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2007-2025 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials are made available under the
 // terms of the Eclipse Public License 2.0 which is available at
 // https://www.eclipse.org/legal/epl-2.0/
@@ -31,6 +31,9 @@
 #include <microsim/MSEventControl.h>
 #include <microsim/MSGlobals.h>
 #include <microsim/MSVehicleControl.h>
+#include <microsim/MSInsertionControl.h>
+#include <microsim/transportables/MSTransportable.h>
+#include <microsim/devices/MSDevice_Taxi.h>
 #include <utils/options/OptionsCont.h>
 #include <utils/common/WrappingCommand.h>
 #include <utils/common/StaticCommand.h>
@@ -40,6 +43,7 @@
 #include <utils/router/AStarRouter.h>
 #include <utils/router/CHRouter.h>
 #include <utils/router/CHRouterWrapper.h>
+#include <utils/vehicle/SUMOVehicleParserHelper.h>
 
 //#define DEBUG_SEPARATE_TURNS
 #define DEBUG_COND(obj) (obj->isSelected())
@@ -60,11 +64,13 @@ SUMOTime MSRoutingEngine::myAdaptationInterval = -1;
 SUMOTime MSRoutingEngine::myLastAdaptation = -1;
 bool MSRoutingEngine::myWithTaz;
 bool MSRoutingEngine::myBikeSpeeds;
-MSRoutingEngine::MSRouterProvider* MSRoutingEngine::myRouterProvider = nullptr;
-std::map<std::pair<const MSEdge*, const MSEdge*>, const MSRoute*> MSRoutingEngine::myCachedRoutes;
+MSRouterProvider* MSRoutingEngine::myRouterProvider = nullptr;
+std::map<std::pair<const MSEdge*, const MSEdge*>, ConstMSRoutePtr> MSRoutingEngine::myCachedRoutes;
 double MSRoutingEngine::myPriorityFactor(0);
 double MSRoutingEngine::myMinEdgePriority(std::numeric_limits<double>::max());
 double MSRoutingEngine::myEdgePriorityRange(0);
+std::map<std::thread::id, SumoRNG*> MSRoutingEngine::myThreadRNGs;
+bool MSRoutingEngine::myHaveRoutingThreads(false);
 
 SUMOAbstractRouter<MSEdge, SUMOVehicle>::Operation MSRoutingEngine::myEffortFunc = &MSRoutingEngine::getEffort;
 #ifdef HAVE_FOX
@@ -92,7 +98,7 @@ MSRoutingEngine::initWeightUpdate() {
             myEdgeWeightSettingCommand = new StaticCommand<MSRoutingEngine>(&MSRoutingEngine::adaptEdgeEfforts);
             MSNet::getInstance()->getEndOfTimestepEvents()->addEvent(myEdgeWeightSettingCommand);
         } else if (period > 0) {
-            WRITE_WARNING("Rerouting is useless if the edge weights do not get updated!");
+            WRITE_WARNING(TL("Rerouting is useless if the edge weights do not get updated!"));
         }
         OutputDevice::createDeviceByOption("device.rerouting.output", "weights", "meandata_file.xsd");
     }
@@ -100,12 +106,19 @@ MSRoutingEngine::initWeightUpdate() {
 
 
 void
-MSRoutingEngine::initEdgeWeights(SUMOVehicleClass svc) {
+MSRoutingEngine::initEdgeWeights(SUMOVehicleClass svc, SUMOTime lastAdaption, int index) {
     if (myBikeSpeeds && svc == SVC_BICYCLE) {
         _initEdgeWeights(myEdgeBikeSpeeds, myPastEdgeBikeSpeeds);
     } else {
         _initEdgeWeights(myEdgeSpeeds, myPastEdgeSpeeds);
     }
+    if (lastAdaption >= 0) {
+        myLastAdaptation = lastAdaption;
+    }
+    if (index >= 0) {
+        assert(index < (int)myPastEdgeSpeeds.size());
+        myAdaptationStepsIndex = index;
+    }
 }
 
 
@@ -144,11 +157,11 @@ MSRoutingEngine::_initEdgeWeights(std::vector<double>& edgeSpeeds, std::vector<s
         myLastAdaptation = MSNet::getInstance()->getCurrentTimeStep();
         myPriorityFactor = oc.getFloat("weights.priority-factor");
         if (myPriorityFactor < 0) {
-            throw ProcessError("weights.priority-factor cannot be negative.");
+            throw ProcessError(TL("weights.priority-factor cannot be negative."));
         }
         if (myPriorityFactor > 0) {
             if (myEdgePriorityRange == 0) {
-                WRITE_WARNING("Option weights.priority-factor does not take effect because all edges have the same priority");
+                WRITE_WARNING(TL("Option weights.priority-factor does not take effect because all edges have the same priority"));
                 myPriorityFactor = 0;
             }
         }
@@ -175,6 +188,16 @@ MSRoutingEngine::getEffortBike(const MSEdge* const e, const SUMOVehicle* const v
     return e->getMinimumTravelTime(v);
 }
 
+SumoRNG*
+MSRoutingEngine::getThreadRNG() {
+    if (myHaveRoutingThreads) {
+        auto it = myThreadRNGs.find(std::this_thread::get_id());
+        // created by InitTask
+        assert(it != myThreadRNGs.end());
+        return it->second;
+    }
+    return nullptr;
+}
 
 
 double
@@ -183,7 +206,7 @@ MSRoutingEngine::getEffortExtra(const MSEdge* const e, const SUMOVehicle* const
                      ? getEffort(e, v, t)
                      : getEffortBike(e, v, t));
     if (gWeightsRandomFactor != 1.) {
-        effort *= RandHelper::rand(1., gWeightsRandomFactor);
+        effort *= RandHelper::rand(1., gWeightsRandomFactor, getThreadRNG());
     }
     if (myPriorityFactor != 0) {
         // lower priority should result in higher effort (and the edge with
@@ -191,6 +214,9 @@ MSRoutingEngine::getEffortExtra(const MSEdge* const e, const SUMOVehicle* const
         const double relativeInversePrio = 1 - ((e->getPriority() - myMinEdgePriority) / myEdgePriorityRange);
         effort *= 1 + relativeInversePrio * myPriorityFactor;
     }
+    if (gRoutingPreferences) {
+        effort /= MSNet::getInstance()->getPreference(e->getRoutingType(), v->getVTypeParameter());
+    }
     return effort;
 }
 
@@ -210,70 +236,52 @@ MSRoutingEngine::adaptEdgeEfforts(SUMOTime currentTime) {
     if (MSNet::getInstance()->getVehicleControl().getDepartedVehicleNo() == 0) {
         return myAdaptationInterval;
     }
-    std::map<std::pair<const MSEdge*, const MSEdge*>, const MSRoute*>::iterator it = myCachedRoutes.begin();
-    for (; it != myCachedRoutes.end(); ++it) {
-        it->second->release();
-    }
     myCachedRoutes.clear();
     const MSEdgeVector& edges = MSNet::getInstance()->getEdgeControl().getEdges();
-    if (myAdaptationSteps > 0) {
-        // moving average
-        for (MSEdgeVector::const_iterator i = edges.begin(); i != edges.end(); ++i) {
-            if ((*i)->isDelayed()) {
-                const int id = (*i)->getNumericalID();
-                double currSpeed = (*i)->getMeanSpeed();
-                if (MSGlobals::gWeightsSeparateTurns > 0 && (*i)->getNumSuccessors() > 1) {
-                    currSpeed = patchSpeedForTurns(*i, currSpeed);
-                }
+    const double newWeightFactor = (double)(1. - myAdaptationWeight);
+    for (const MSEdge* const e : edges) {
+        if (e->isDelayed()) {
+            const int id = e->getNumericalID();
+            double currSpeed = e->getMeanSpeed();
+            if (MSGlobals::gWeightsSeparateTurns > 0 && e->getNumSuccessors() > 1) {
+                currSpeed = patchSpeedForTurns(e, currSpeed);
+            }
 #ifdef DEBUG_SEPARATE_TURNS
-                if (DEBUG_COND((*i)->getLanes()[0])) {
-                    std::cout << SIMTIME << " edge=" << (*i)->getID()
-                              << " meanSpeed=" << (*i)->getMeanSpeed()
-                              << " currSpeed=" << currSpeed
-                              << " oldestSpeed=" << myPastEdgeSpeeds[id][myAdaptationStepsIndex]
-                              << " oldAvg=" << myEdgeSpeeds[id]
-                              << "\n";
-                }
+            if (DEBUG_COND(e->getLanes()[0])) {
+                std::cout << SIMTIME << " edge=" << e->getID()
+                          << " meanSpeed=" << e->getMeanSpeed()
+                          << " currSpeed=" << currSpeed
+                          << " oldestSpeed=" << myPastEdgeSpeeds[id][myAdaptationStepsIndex]
+                          << " oldAvg=" << myEdgeSpeeds[id]
+                          << "\n";
+            }
 #endif
+            if (myAdaptationSteps > 0) {
+                // moving average
                 myEdgeSpeeds[id] += (currSpeed - myPastEdgeSpeeds[id][myAdaptationStepsIndex]) / myAdaptationSteps;
                 myPastEdgeSpeeds[id][myAdaptationStepsIndex] = currSpeed;
-            }
-        }
-        if (myBikeSpeeds) {
-            for (MSEdgeVector::const_iterator i = edges.begin(); i != edges.end(); ++i) {
-                if ((*i)->isDelayed()) {
-                    const int id = (*i)->getNumericalID();
-                    const double currSpeed = (*i)->getMeanSpeedBike();
-                    myEdgeBikeSpeeds[id] += (currSpeed - myPastEdgeBikeSpeeds[id][myAdaptationStepsIndex]) / myAdaptationSteps;
-                    myPastEdgeBikeSpeeds[id][myAdaptationStepsIndex] = currSpeed;
+                if (myBikeSpeeds) {
+                    const double currBikeSpeed = e->getMeanSpeedBike();
+                    myEdgeBikeSpeeds[id] += (currBikeSpeed - myPastEdgeBikeSpeeds[id][myAdaptationStepsIndex]) / myAdaptationSteps;
+                    myPastEdgeBikeSpeeds[id][myAdaptationStepsIndex] = currBikeSpeed;
                 }
-            }
-        }
-        myAdaptationStepsIndex = (myAdaptationStepsIndex + 1) % myAdaptationSteps;
-    } else {
-        // exponential moving average
-        const double newWeightFactor = (double)(1. - myAdaptationWeight);
-        for (MSEdgeVector::const_iterator i = edges.begin(); i != edges.end(); ++i) {
-            if ((*i)->isDelayed()) {
-                const int id = (*i)->getNumericalID();
-                const double currSpeed = (*i)->getMeanSpeed();
+            } else {
+                // exponential moving average
                 if (currSpeed != myEdgeSpeeds[id]) {
                     myEdgeSpeeds[id] = myEdgeSpeeds[id] * myAdaptationWeight + currSpeed * newWeightFactor;
                 }
-            }
-        }
-        if (myBikeSpeeds) {
-            for (MSEdgeVector::const_iterator i = edges.begin(); i != edges.end(); ++i) {
-                if ((*i)->isDelayed()) {
-                    const int id = (*i)->getNumericalID();
-                    const double currSpeed = (*i)->getMeanSpeedBike();
-                    if (currSpeed != myEdgeBikeSpeeds[id]) {
-                        myEdgeBikeSpeeds[id] = myEdgeBikeSpeeds[id] * myAdaptationWeight + currSpeed * newWeightFactor;
+                if (myBikeSpeeds) {
+                    const double currBikeSpeed = e->getMeanSpeedBike();
+                    if (currBikeSpeed != myEdgeBikeSpeeds[id]) {
+                        myEdgeBikeSpeeds[id] = myEdgeBikeSpeeds[id] * myAdaptationWeight + currBikeSpeed * newWeightFactor;
                     }
                 }
             }
         }
     }
+    if (myAdaptationSteps > 0) {
+        myAdaptationStepsIndex = (myAdaptationStepsIndex + 1) % myAdaptationSteps;
+    }
     myLastAdaptation = currentTime + DELTA_T; // because we run at the end of the time step
     if (OptionsCont::getOptions().isSet("device.rerouting.output")) {
         OutputDevice& dev = OutputDevice::getDeviceByOption("device.rerouting.output");
@@ -302,6 +310,9 @@ MSRoutingEngine::patchSpeedForTurns(const MSEdge* edge, double currSpeed) {
     const double length = edge->getLength();
     double maxSpeed = 0;
     for (const auto& pair : edge->getViaSuccessors()) {
+        if (pair.second == nullptr) {
+            continue;
+        }
         TimeAndCount& tc = myEdgeTravelTimes[pair.second->getNumericalID()];
         if (tc.second > 0) {
             const double avgSpeed = length / STEPS2TIME(tc.first / tc.second);
@@ -312,6 +323,9 @@ MSRoutingEngine::patchSpeedForTurns(const MSEdge* edge, double currSpeed) {
         // perform correction
         const double correctedSpeed = MSGlobals::gWeightsSeparateTurns * maxSpeed + (1 - MSGlobals::gWeightsSeparateTurns) * currSpeed;
         for (const auto& pair : edge->getViaSuccessors()) {
+            if (pair.second == nullptr) {
+                continue;
+            }
             const int iid = pair.second->getNumericalID();
             TimeAndCount& tc = myEdgeTravelTimes[iid];
             if (tc.second > 0) {
@@ -359,7 +373,7 @@ MSRoutingEngine::patchSpeedForTurns(const MSEdge* edge, double currSpeed) {
 }
 
 
-const MSRoute*
+ConstMSRoutePtr
 MSRoutingEngine::getCachedRoute(const std::pair<const MSEdge*, const MSEdge*>& key) {
     auto routeIt = myCachedRoutes.find(key);
     if (routeIt != myCachedRoutes.end()) {
@@ -375,13 +389,13 @@ MSRoutingEngine::initRouter(SUMOVehicle* vehicle) {
     const std::string routingAlgorithm = oc.getString("routing-algorithm");
     const bool hasPermissions = MSNet::getInstance()->hasPermissions();
     myBikeSpeeds = oc.getBool("device.rerouting.bike-speeds");
-    myEffortFunc = ((gWeightsRandomFactor != 1 || myPriorityFactor != 0 || myBikeSpeeds) ? &MSRoutingEngine::getEffortExtra : &MSRoutingEngine::getEffort);
+    myEffortFunc = ((gWeightsRandomFactor != 1 || myPriorityFactor != 0 || myBikeSpeeds || gRoutingPreferences) ? &MSRoutingEngine::getEffortExtra : &MSRoutingEngine::getEffort);
 
     SUMOAbstractRouter<MSEdge, SUMOVehicle>* router = nullptr;
     if (routingAlgorithm == "dijkstra") {
         router = new DijkstraRouter<MSEdge, SUMOVehicle>(MSEdge::getAllEdges(), true, myEffortFunc, nullptr, false, nullptr, true);
     } else if (routingAlgorithm == "astar") {
-        typedef AStarRouter<MSEdge, SUMOVehicle> AStar;
+        typedef AStarRouter<MSEdge, SUMOVehicle, MSMapMatcher> AStar;
         std::shared_ptr<const AStar::LookupTable> lookup = nullptr;
         if (oc.isSet("astar.all-distances")) {
             lookup = std::make_shared<const AStar::FLT>(oc.getString("astar.all-distances"), (int)MSEdge::getAllEdges().size());
@@ -393,7 +407,7 @@ MSRoutingEngine::initRouter(SUMOVehicle* vehicle) {
                 MSEdge::getAllEdges(), true, &MSNet::getTravelTime,
                 string2time(oc.getString("begin")), string2time(oc.getString("end")), SUMOTime_MAX, hasPermissions, 1);
             lookup = std::make_shared<const AStar::LMLT>(oc.getString("astar.landmark-distances"), MSEdge::getAllEdges(), &chrouter,
-                     nullptr, vehicle, "", oc.getInt("device.rerouting.threads"));
+                     nullptr, vehicle, "", oc.getInt("device.rerouting.threads"), MSNet::getInstance()->getMapMatcher());
             vehicle->setChosenSpeedFactor(speedFactor);
         }
         router = new AStar(MSEdge::getAllEdges(), true, myEffortFunc, lookup, true);
@@ -408,24 +422,51 @@ MSRoutingEngine::initRouter(SUMOVehicle* vehicle) {
             MSEdge::getAllEdges(), true, myEffortFunc,
             string2time(oc.getString("begin")), string2time(oc.getString("end")), weightPeriod, hasPermissions, oc.getInt("device.rerouting.threads"));
     } else {
-        throw ProcessError("Unknown routing algorithm '" + routingAlgorithm + "'!");
+        throw ProcessError(TLF("Unknown routing algorithm '%'!", routingAlgorithm));
     }
 
     RailwayRouter<MSEdge, SUMOVehicle>* railRouter = nullptr;
     if (MSNet::getInstance()->hasBidiEdges()) {
-        railRouter = new RailwayRouter<MSEdge, SUMOVehicle>(MSEdge::getAllEdges(), true, myEffortFunc, nullptr, false, true, false, oc.getFloat("railway.max-train-length"));
+        railRouter = new RailwayRouter<MSEdge, SUMOVehicle>(MSEdge::getAllEdges(), true, myEffortFunc, nullptr, false, true, false,
+                oc.getFloat("railway.max-train-length"),
+                oc.getFloat("weights.reversal-penalty"));
     }
-    myRouterProvider = new MSRouterProvider(router, nullptr, nullptr, railRouter);
+    const int carWalk = SUMOVehicleParserHelper::parseCarWalkTransfer(oc, MSDevice_Taxi::hasFleet() || MSNet::getInstance()->getInsertionControl().hasTaxiFlow());
+    const double taxiWait = STEPS2TIME(string2time(OptionsCont::getOptions().getString("persontrip.taxi.waiting-time")));
+    MSTransportableRouter* transRouter = new MSTransportableRouter(MSNet::adaptIntermodalRouter, carWalk, taxiWait, routingAlgorithm, 0);
+    myRouterProvider = new MSRouterProvider(router, nullptr, transRouter, railRouter);
 #ifndef THREAD_POOL
 #ifdef HAVE_FOX
-    FXWorkerThread::Pool& threadPool = MSNet::getInstance()->getEdgeControl().getThreadPool();
+    MFXWorkerThread::Pool& threadPool = MSNet::getInstance()->getEdgeControl().getThreadPool();
     if (threadPool.size() > 0) {
-        const std::vector<FXWorkerThread*>& threads = threadPool.getWorkers();
+        const std::vector<MFXWorkerThread*>& threads = threadPool.getWorkers();
         if (static_cast<MSEdgeControl::WorkerThread*>(threads.front())->setRouterProvider(myRouterProvider)) {
-            for (std::vector<FXWorkerThread*>::const_iterator t = threads.begin() + 1; t != threads.end(); ++t) {
+            for (std::vector<MFXWorkerThread*>::const_iterator t = threads.begin() + 1; t != threads.end(); ++t) {
                 static_cast<MSEdgeControl::WorkerThread*>(*t)->setRouterProvider(myRouterProvider->clone());
             }
         }
+        myHaveRoutingThreads = true;
+        for (int i = 0; i < threadPool.size(); i++) {
+            threadPool.add(new InitTask(), i);
+        }
+        threadPool.waitAll();
+        // to use when routing is triggered from the main thread (i.e. by a rerouter)
+        myThreadRNGs[std::this_thread::get_id()] = nullptr;
+    }
+#endif
+#endif
+}
+
+
+void
+MSRoutingEngine::initGUIThreadRNG() {
+#ifndef THREAD_POOL
+#ifdef HAVE_FOX
+    MFXWorkerThread::Pool& threadPool = MSNet::getInstance()->getEdgeControl().getThreadPool();
+    if (threadPool.size() > 0) {
+        FXMutexLock lock(myRouteCacheMutex);
+        SumoRNG* rng = new SumoRNG("routingGUI");
+        myThreadRNGs[std::this_thread::get_id()] = rng;
     }
 #endif
 #endif
@@ -434,14 +475,14 @@ MSRoutingEngine::initRouter(SUMOVehicle* vehicle) {
 
 void
 MSRoutingEngine::reroute(SUMOVehicle& vehicle, const SUMOTime currentTime, const std::string& info,
-                         const bool onInit, const bool silent, const MSEdgeVector& prohibited) {
+                         const bool onInit, const bool silent, const Prohibitions& prohibited) {
     if (myRouterProvider == nullptr) {
         initRouter(&vehicle);
     }
     auto& router = myRouterProvider->getVehicleRouter(vehicle.getVClass());
 #ifndef THREAD_POOL
 #ifdef HAVE_FOX
-    FXWorkerThread::Pool& threadPool = MSNet::getInstance()->getEdgeControl().getThreadPool();
+    MFXWorkerThread::Pool& threadPool = MSNet::getInstance()->getEdgeControl().getThreadPool();
     if (threadPool.size() > 0) {
         threadPool.add(new RoutingTask(vehicle, currentTime, info, onInit, silent, prohibited));
         return;
@@ -456,13 +497,45 @@ MSRoutingEngine::reroute(SUMOVehicle& vehicle, const SUMOTime currentTime, const
     } catch (ProcessError&) {
         if (!silent) {
             if (!prohibited.empty()) {
-                router.prohibit(MSEdgeVector());
+                router.prohibit(Prohibitions());
             }
             throw;
         }
     }
     if (!prohibited.empty()) {
-        router.prohibit(MSEdgeVector());
+        router.prohibit(Prohibitions());
+    }
+}
+
+
+void
+MSRoutingEngine::reroute(MSTransportable& t, const SUMOTime currentTime, const std::string& info,
+                         const bool onInit, const bool silent, const Prohibitions& prohibited) {
+    MSTransportableRouter& router = getIntermodalRouterTT(t.getRNGIndex(), prohibited);
+#ifndef THREAD_POOL
+#ifdef HAVE_FOX
+    MFXWorkerThread::Pool& threadPool = MSNet::getInstance()->getEdgeControl().getThreadPool();
+    if (threadPool.size() > 0) {
+        // threadPool.add(new RoutingTask(t, currentTime, info, onInit, silent, prohibited));
+        return;
+    }
+#endif
+#endif
+    if (!prohibited.empty()) {
+        router.prohibit(prohibited);
+    }
+    try {
+        t.reroute(currentTime, info, router, onInit, myWithTaz, silent);
+    } catch (ProcessError&) {
+        if (!silent) {
+            if (!prohibited.empty()) {
+                router.prohibit(Prohibitions());
+            }
+            throw;
+        }
+    }
+    if (!prohibited.empty()) {
+        router.prohibit(Prohibitions());
     }
 }
 
@@ -480,8 +553,8 @@ MSRoutingEngine::addEdgeTravelTime(const MSEdge& edge, const SUMOTime travelTime
 }
 
 
-SUMOAbstractRouter<MSEdge, SUMOVehicle>&
-MSRoutingEngine::getRouterTT(const int rngIndex, SUMOVehicleClass svc, const MSEdgeVector& prohibited) {
+MSVehicleRouter&
+MSRoutingEngine::getRouterTT(const int rngIndex, SUMOVehicleClass svc, const Prohibitions& prohibited) {
     if (myRouterProvider == nullptr) {
         initWeightUpdate();
         initEdgeWeights(svc);
@@ -489,12 +562,14 @@ MSRoutingEngine::getRouterTT(const int rngIndex, SUMOVehicleClass svc, const MSE
     }
 #ifndef THREAD_POOL
 #ifdef HAVE_FOX
-    FXWorkerThread::Pool& threadPool = MSNet::getInstance()->getEdgeControl().getThreadPool();
+    MFXWorkerThread::Pool& threadPool = MSNet::getInstance()->getEdgeControl().getThreadPool();
     if (threadPool.size() > 0) {
         auto& router = static_cast<MSEdgeControl::WorkerThread*>(threadPool.getWorkers()[rngIndex % MSGlobals::gNumThreads])->getRouter(svc);
         router.prohibit(prohibited);
         return router;
     }
+#else
+    UNUSED_PARAMETER(rngIndex);
 #endif
 #endif
     myRouterProvider->getVehicleRouter(svc).prohibit(prohibited);
@@ -502,6 +577,30 @@ MSRoutingEngine::getRouterTT(const int rngIndex, SUMOVehicleClass svc, const MSE
 }
 
 
+MSTransportableRouter&
+MSRoutingEngine::getIntermodalRouterTT(const int rngIndex, const Prohibitions& prohibited) {
+    if (myRouterProvider == nullptr) {
+        initWeightUpdate();
+        initEdgeWeights(SVC_PEDESTRIAN);
+        initRouter();
+    }
+#ifndef THREAD_POOL
+#ifdef HAVE_FOX
+    MFXWorkerThread::Pool& threadPool = MSNet::getInstance()->getEdgeControl().getThreadPool();
+    if (threadPool.size() > 0) {
+        auto& router = static_cast<MSEdgeControl::WorkerThread*>(threadPool.getWorkers()[rngIndex % MSGlobals::gNumThreads])->getIntermodalRouter();
+        router.prohibit(prohibited);
+        return router;
+    }
+#else
+    UNUSED_PARAMETER(rngIndex);
+#endif
+#endif
+    myRouterProvider->getIntermodalRouter().prohibit(prohibited);
+    return myRouterProvider->getIntermodalRouter();
+}
+
+
 void
 MSRoutingEngine::cleanup() {
     myAdaptationInterval = -1; // responsible for triggering initEdgeWeights
@@ -528,11 +627,76 @@ MSRoutingEngine::cleanup() {
 }
 
 
+void
+MSRoutingEngine::saveState(OutputDevice& out) {
+    if (myEdgeSpeeds.size() == 0) {
+        return;
+    }
+    out.openTag(SUMO_TAG_ROUTINGENGINE);
+    out.writeAttr(SUMO_ATTR_LAST, myLastAdaptation);
+    out.writeAttr(SUMO_ATTR_INDEX, myAdaptationStepsIndex);
+    const MSEdgeVector& edges = MSNet::getInstance()->getEdgeControl().getEdges();
+    for (const MSEdge* const e : edges) {
+        if (e->isDelayed()) {
+            const int id = e->getNumericalID();
+            out.openTag(SUMO_TAG_EDGE);
+            out.writeAttr(SUMO_ATTR_ID, e->getID());
+            out.writeAttr(SUMO_ATTR_SPEED, myEdgeSpeeds[id]);
+            if (myAdaptationSteps > 0) {
+                out.writeAttr(SUMO_ATTR_PASTSPEED, myPastEdgeSpeeds[id]);
+            }
+            if (myBikeSpeeds) {
+                out.writeAttr(SUMO_ATTR_BIKESPEED, myEdgeBikeSpeeds[id]);
+                if (myAdaptationSteps > 0) {
+                    out.writeAttr(SUMO_ATTR_PASTBIKESPEED, myPastEdgeBikeSpeeds[id]);
+                }
+            }
+            out.closeTag();
+        }
+    }
+    out.closeTag();
+}
+
+
+void
+MSRoutingEngine::loadState(const SUMOSAXAttributes& attrs) {
+    const MSEdge* const e = MSEdge::dictionary(attrs.getString(SUMO_ATTR_ID));
+    e->markDelayed();
+    const int id = e->getNumericalID();
+    bool checkedSteps = false;
+    bool checkedBikeSpeeds = false;
+    bool ok = true;
+    if ((int)myEdgeSpeeds.size() > id) {
+        myEdgeSpeeds[id] = attrs.get<double>(SUMO_ATTR_SPEED, nullptr, ok);
+        if (myBikeSpeeds) {
+            if (attrs.hasAttribute(SUMO_ATTR_BIKESPEED)) {
+                myEdgeBikeSpeeds[id] = attrs.get<double>(SUMO_ATTR_BIKESPEED, nullptr, ok);
+            } else if (!checkedBikeSpeeds) {
+                checkedBikeSpeeds = true;
+                WRITE_WARNING("Bike speeds missing in loaded state");
+            }
+        }
+        if (myAdaptationSteps > 0) {
+            const std::vector<double> speeds = attrs.getOpt<std::vector<double> >(SUMO_ATTR_PASTSPEED, nullptr, ok);
+            if ((int)speeds.size() == myAdaptationSteps) {
+                myPastEdgeSpeeds[id] = speeds;
+                if (myBikeSpeeds && attrs.hasAttribute(SUMO_ATTR_PASTBIKESPEED)) {
+                    myPastEdgeBikeSpeeds[id] = attrs.getOpt<std::vector<double> >(SUMO_ATTR_PASTBIKESPEED, nullptr, ok);
+                }
+            } else if (!checkedSteps) {
+                checkedSteps = true;
+                WRITE_WARNING("Number of adaptation speeds in loaded state doesn't match option --device.rerouting.adaptation-steps");
+            }
+        }
+    }
+}
+
+
 #ifdef HAVE_FOX
 void
 MSRoutingEngine::waitForAll() {
 #ifndef THREAD_POOL
-    FXWorkerThread::Pool& threadPool = MSNet::getInstance()->getEdgeControl().getThreadPool();
+    MFXWorkerThread::Pool& threadPool = MSNet::getInstance()->getEdgeControl().getThreadPool();
     if (threadPool.size() > 0) {
         threadPool.waitAll();
     }
@@ -544,7 +708,7 @@ MSRoutingEngine::waitForAll() {
 // MSRoutingEngine::RoutingTask-methods
 // ---------------------------------------------------------------------------
 void
-MSRoutingEngine::RoutingTask::run(FXWorkerThread* context) {
+MSRoutingEngine::RoutingTask::run(MFXWorkerThread* context) {
     SUMOAbstractRouter<MSEdge, SUMOVehicle>& router = static_cast<MSEdgeControl::WorkerThread*>(context)->getRouter(myVehicle.getVClass());
     if (!myProhibited.empty()) {
         router.prohibit(myProhibited);
@@ -554,13 +718,13 @@ MSRoutingEngine::RoutingTask::run(FXWorkerThread* context) {
     } catch (ProcessError&) {
         if (!mySilent) {
             if (!myProhibited.empty()) {
-                router.prohibit(MSEdgeVector());
+                router.prohibit(Prohibitions());
             }
             throw;
         }
     }
     if (!myProhibited.empty()) {
-        router.prohibit(MSEdgeVector());
+        router.prohibit(Prohibitions());
     }
     const MSEdge* source = *myVehicle.getRoute().begin();
     const MSEdge* dest = myVehicle.getRoute().getLastEdge();
@@ -568,11 +732,21 @@ MSRoutingEngine::RoutingTask::run(FXWorkerThread* context) {
         const std::pair<const MSEdge*, const MSEdge*> key = std::make_pair(source, dest);
         FXMutexLock lock(myRouteCacheMutex);
         if (MSRoutingEngine::myCachedRoutes.find(key) == MSRoutingEngine::myCachedRoutes.end()) {
-            MSRoutingEngine::myCachedRoutes[key] = &myVehicle.getRoute();
-            myVehicle.getRoute().addReference();
+            MSRoutingEngine::myCachedRoutes[key] = myVehicle.getRoutePtr();
         }
     }
 }
+
+// ---------------------------------------------------------------------------
+// MSRoutingEngine::InitTask-methods
+// ---------------------------------------------------------------------------
+void
+MSRoutingEngine::InitTask::run(MFXWorkerThread* /*context*/) {
+    FXMutexLock lock(myRouteCacheMutex);
+    SumoRNG* rng = new SumoRNG("routing_" + toString(myThreadRNGs.size()));
+    myThreadRNGs[std::this_thread::get_id()] = rng;
+}
+
 #endif
 
 
