@@ -1,11 +1,15 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
-// This program and the accompanying materials
-// are made available under the terms of the Eclipse Public License v2.0
-// which accompanies this distribution, and is available at
-// http://www.eclipse.org/legal/epl-v20.html
-// SPDX-License-Identifier: EPL-2.0
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2001-2025 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 /****************************************************************************/
 /// @file    MSLaneSpeedTrigger.h
 /// @author  Daniel Krajzewicz
@@ -16,21 +20,15 @@
 ///
 // Changes the speed allowed on a set of lanes
 /****************************************************************************/
-#ifndef MSLaneSpeedTrigger_h
-#define MSLaneSpeedTrigger_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
+#pragma once
 #include <config.h>
 
 #include <string>
 #include <vector>
 #include <utils/common/Command.h>
+#include <utils/common/Named.h>
 #include <utils/common/UtilExceptions.h>
 #include <utils/xml/SUMOSAXHandler.h>
-#include "MSTrigger.h"
 
 
 // ===========================================================================
@@ -50,7 +48,7 @@ class MSLane;
  * responsible for from a file and sets it.
  * Lanes with variable speeds are so possible.
  */
-class MSLaneSpeedTrigger : public MSTrigger, public SUMOSAXHandler, public Command {
+class MSLaneSpeedTrigger : public Named, public SUMOSAXHandler {
 public:
     /** @brief Constructor
      *
@@ -75,8 +73,9 @@ public:
      * @see Command
      * @see MSEventControl
      */
-    SUMOTime execute(SUMOTime currentTime);
+    SUMOTime executeSpeedChange(SUMOTime currentTime);
 
+    SUMOTime executeFrictionChange(SUMOTime currentTime);
 
     SUMOTime processCommand(bool move2next, SUMOTime currentTime);
 
@@ -91,6 +90,18 @@ public:
     /// Returns the current speed
     double getCurrentSpeed() const;
 
+    /// Returns the current friction
+    double getCurrentFriction() const;
+
+    /// @brief return all MSLaneSpeedTrigger instances
+    static const std::map<std::string, MSLaneSpeedTrigger*>& getInstances() {
+        return myInstances;
+    }
+
+    const std::vector<MSLane*>& getLanes() {
+        return myDestLanes;
+    }
+
 protected:
     /// @name inherited from GenericSAXHandler
     //@{
@@ -119,11 +130,11 @@ protected:
     /** the lane the trigger is responsible for */
     std::vector<MSLane*> myDestLanes;
 
-    /** the speed that will be set on the next call */
-    double myCurrentSpeed;
-
     /// The original speed allowed on the lanes
-    double myDefaultSpeed;
+    const double myDefaultSpeed;
+
+    /// The original friction on the lanes
+    const double myDefaultFriction;
 
     /// The information whether the read speed shall be overridden
     bool myAmOverriding;
@@ -132,7 +143,10 @@ protected:
     double mySpeedOverrideValue;
 
     std::vector<std::pair<SUMOTime, double> > myLoadedSpeeds;
-    std::vector<std::pair<SUMOTime, double> >::iterator myCurrentEntry;
+    std::vector<std::pair<SUMOTime, double> > myLoadedFrictions;
+    std::vector<std::pair<SUMOTime, double> >::iterator myCurrentSpeedEntry;
+    std::vector<std::pair<SUMOTime, double> >::iterator myCurrentFrictionEntry;
+    static std::map<std::string, MSLaneSpeedTrigger*> myInstances;
 
 private:
     /// @brief Invalidated copy constructor.
@@ -149,9 +163,3 @@ private:
     bool myDidInit;
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
