@@ -37,7 +37,7 @@ netedit.checkUndoRedo(referencePosition)
 
 # save TAZs
 # save Netedit config
-netedit.saveExistentShortcut("neteditConfig")
+netedit.saveExistentFile("neteditConfig")
 
 # quit netedit
 netedit.quit(neteditProcess)
